@@ -73,10 +73,8 @@ static void uri_removed_cb (DBusGProxy *proxy,
                             BrQueue    *queue);
 static void now_playing_changed (DBusGProxy *proxy,
                                  const char *uri,
-                                 int         type,
                                  BrQueue    *queue);
 static void position_changed (DBusGProxy *proxy,
-                              int         type,
                               double      position,
                               BrQueue    *queue);
 static void playing_changed (DBusGProxy *proxy,
@@ -129,9 +127,9 @@ br_queue_dispose (GObject *object)
 
 static void
 br_queue_set_property (GObject      *object,
-                          guint         prop_id,
-                          const GValue *value,
-                          GParamSpec   *pspec)
+                       guint         prop_id,
+                       const GValue *value,
+                       GParamSpec   *pspec)
 {
     BrQueue *self = (BrQueue *) object;
     BrQueuePrivate *priv = self->priv;
@@ -153,9 +151,9 @@ br_queue_set_property (GObject      *object,
 
 static void
 br_queue_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
+                       guint       prop_id,
+                       GValue     *value,
+                       GParamSpec *pspec)
 {
     BrQueue *self = (BrQueue *) object;
 
@@ -187,19 +185,17 @@ uri_removed_cb (DBusGProxy *proxy,
 static void
 now_playing_changed (DBusGProxy *proxy,
                      const char *uri,
-                     int         type,
                      BrQueue    *queue)
 {
-    g_signal_emit (queue, signals[NOW_PLAYING], 0, uri, type);
+    g_signal_emit (queue, signals[NOW_PLAYING], 0, uri);
 }
 
 static void
 position_changed (DBusGProxy *proxy,
-                  int         type,
                   double      position,
                   BrQueue    *queue)
 {
-    g_signal_emit (queue, signals[POSITION_CHANGED], 0, type, position);
+    g_signal_emit (queue, signals[POSITION_CHANGED], 0, position);
 }
 
 static void
@@ -260,9 +256,9 @@ br_queue_constructor (GType                  type,
     dbus_g_proxy_add_signal (priv->proxy, "UriRemoved",
                              G_TYPE_STRING, G_TYPE_INT, G_TYPE_INVALID);
     dbus_g_proxy_add_signal (priv->proxy, "NowPlayingChanged",
-                             G_TYPE_STRING, G_TYPE_INT, G_TYPE_INVALID);
+                             G_TYPE_STRING, G_TYPE_INVALID);
     dbus_g_proxy_add_signal (priv->proxy, "PositionChanged",
-                             G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_INVALID);
+                             G_TYPE_DOUBLE, G_TYPE_INVALID);
     dbus_g_proxy_add_signal (priv->proxy, "PlayingChanged",
                              G_TYPE_BOOLEAN, G_TYPE_INVALID);
     dbus_g_proxy_connect_signal (priv->proxy, "UriAdded",
@@ -295,9 +291,12 @@ br_queue_class_init (BrQueueClass *klass)
     dbus_g_object_register_marshaller (br_marshal_VOID__STRING_INT,
                                        G_TYPE_NONE, G_TYPE_STRING,
                                        G_TYPE_INT, G_TYPE_INVALID);
+#if 0
     dbus_g_object_register_marshaller (br_marshal_VOID__INT_DOUBLE,
                                        G_TYPE_NONE, G_TYPE_INT,
                                        G_TYPE_DOUBLE, G_TYPE_INVALID);
+#endif
+
     g_object_class_install_property (o_class, PROP_PATH,
                                      g_param_spec_string ("object-path",
                                                           "", "", "",
@@ -329,17 +328,15 @@ br_queue_class_init (BrQueueClass *klass)
                                          G_TYPE_FROM_CLASS (klass),
                                          G_SIGNAL_RUN_FIRST |
                                          G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
-                                         br_marshal_VOID__STRING_INT,
-                                         G_TYPE_NONE, 2, G_TYPE_STRING,
-                                         G_TYPE_INT);
+                                         g_cclosure_marshal_VOID__STRING,
+                                         G_TYPE_NONE, 1, G_TYPE_STRING);
     signals[POSITION_CHANGED] = g_signal_new ("position-changed",
                                               G_TYPE_FROM_CLASS (klass),
                                               G_SIGNAL_RUN_FIRST |
                                               G_SIGNAL_NO_RECURSE,
                                               0, NULL, NULL,
-                                              br_marshal_VOID__INT_DOUBLE,
-                                              G_TYPE_NONE, 2, G_TYPE_INT,
-                                              G_TYPE_DOUBLE);
+                                              g_cclosure_marshal_VOID__DOUBLE,
+                                              G_TYPE_NONE, 1, G_TYPE_DOUBLE);
     signals[PLAYING_CHANGED] = g_signal_new ("playing-changed",
                                              G_TYPE_FROM_CLASS (klass),
                                              G_SIGNAL_RUN_FIRST |
@@ -431,6 +428,7 @@ async_reply (DBusGProxy *proxy,
 {
     if (error != NULL) {
         g_warning ("Error talking to Bognor-Regis : %s", error->message);
+        g_error_free (error);
     }
 }
 
@@ -461,6 +459,20 @@ br_queue_play_uri (BrQueue    *queue,
                                                  mimetype, async_reply, queue);
 }
 
+void
+br_queue_play_index (BrQueue *queue,
+                     int      index)
+{
+    BrQueuePrivate *priv;
+
+    g_return_if_fail (IS_BR_QUEUE (queue));
+
+    priv = queue->priv;
+
+    org_moblin_BognorRegis_Queue_play_index_async (priv->proxy, index,
+                                                   async_reply, queue);
+}
+
 void
 br_queue_stop (BrQueue *queue)
 {
@@ -553,11 +565,18 @@ list_uris_reply (DBusGProxy *proxy,
     BrQueueListUrisCallback cb;
 
     cb = (BrQueueListUrisCallback) (data->cb);
+
+    if (error) {
+        /* Initialise the parameters correctly */
+        OUT_uris = NULL;
+    }
+
     cb (data->queue, OUT_uris, error, data->userdata);
 
-    g_strfreev (OUT_uris);
     if (error) {
         g_error_free (error);
+    } else {
+        g_strfreev (OUT_uris);
     }
 
     g_free (data);
@@ -586,8 +605,7 @@ br_queue_list_uris (BrQueue                *queue,
 
 static void
 get_now_playing_reply (DBusGProxy *proxy,
-                       char       *OUT_audio_uri,
-                       char       *OUT_visual_uri,
+                       char       *OUT_uri,
                        GError     *error,
                        gpointer    userdata)
 {
@@ -595,13 +613,18 @@ get_now_playing_reply (DBusGProxy *proxy,
     BrQueueGetNowPlayingCallback cb;
 
     cb = (BrQueueGetNowPlayingCallback) (data->cb);
-    cb (data->queue, OUT_audio_uri, OUT_visual_uri, error, data->userdata);
+
+    if (error) {
+        OUT_uri = NULL;
+    }
+
+    cb (data->queue, OUT_uri, error, data->userdata);
 
     if (error) {
         g_error_free (error);
+    } else {
+        g_free (OUT_uri);
     }
-    g_free (OUT_audio_uri);
-    g_free (OUT_visual_uri);
 
     g_free (data);
 }
@@ -638,11 +661,17 @@ get_name_reply (DBusGProxy *proxy,
     BrQueueGetNameCallback cb;
 
     cb = (BrQueueGetNameCallback) (data->cb);
+
+    if (error) {
+        OUT_name = NULL;
+    }
+
     cb (data->queue, OUT_name, error, data->userdata);
 
-    g_free (OUT_name);
     if (error) {
         g_error_free (error);
+    } else {
+        g_free (OUT_name);
     }
 
     g_free (data);
@@ -671,7 +700,6 @@ br_queue_get_name (BrQueue               *queue,
 
 void
 br_queue_set_position (BrQueue    *queue,
-                       BrQueueType type,
                        double      position)
 {
     BrQueuePrivate *priv;
@@ -679,8 +707,7 @@ br_queue_set_position (BrQueue    *queue,
     g_return_if_fail (IS_BR_QUEUE (queue));
 
     priv = queue->priv;
-    org_moblin_BognorRegis_Queue_set_position_async (priv->proxy, type,
-                                                     position,
+    org_moblin_BognorRegis_Queue_set_position_async (priv->proxy, position,
                                                      async_reply, queue);
 }
 
@@ -694,6 +721,11 @@ get_position_reply (DBusGProxy *proxy,
     BrQueueGetPositionCallback cb;
 
     cb = (BrQueueGetPositionCallback) (data->cb);
+
+    if (error) {
+        OUT_position = 0.0;
+    }
+
     cb (data->queue, OUT_position, error, data->userdata);
 
     if (error) {
@@ -705,7 +737,6 @@ get_position_reply (DBusGProxy *proxy,
 
 void
 br_queue_get_position (BrQueue                   *queue,
-                       BrQueueType                type,
                        BrQueueGetPositionCallback cb,
                        gpointer                   userdata)
 {
@@ -721,7 +752,7 @@ br_queue_get_position (BrQueue                   *queue,
     data->cb = G_CALLBACK (cb);
     data->userdata = userdata;
 
-    org_moblin_BognorRegis_Queue_get_position_async (priv->proxy, type,
+    org_moblin_BognorRegis_Queue_get_position_async (priv->proxy,
                                                      get_position_reply, data);
 }
 
@@ -735,6 +766,11 @@ get_playing_reply (DBusGProxy *proxy,
     BrQueueGetPlayingCallback cb;
 
     cb = (BrQueueGetPlayingCallback) (data->cb);
+
+    if (error) {
+        OUT_playing = FALSE;
+    }
+
     cb (data->queue, OUT_playing, error, data->userdata);
 
     if (error) {
