@@ -59,8 +59,7 @@ typedef void (*BrQueueListUrisCallback) (BrQueue *queue,
                                          GError  *error,
                                          gpointer userdata);
 typedef void (*BrQueueGetNowPlayingCallback) (BrQueue *queue,
-                                              char    *audio_uri,
-                                              char    *visual_uri,
+                                              char    *uri,
                                               GError  *error,
                                               gpointer userdata);
 typedef void (*BrQueueGetNameCallback) (BrQueue *queue,
@@ -76,11 +75,6 @@ typedef void (*BrQueueGetPlayingCallback) (BrQueue *queue,
                                            GError  *error,
                                            gpointer userdata);
 
-typedef enum _BrQueueType {
-    BR_QUEUE_AUDIO_TYPE,
-    BR_QUEUE_VISUAL_TYPE
-} BrQueueType;
-
 struct _BrQueue
 {
     GObject parent;
@@ -103,6 +97,8 @@ void br_queue_play (BrQueue *queue);
 void br_queue_play_uri (BrQueue    *queue,
                         const char *uri,
                         const char *mimetype);
+void br_queue_play_index (BrQueue *queue,
+                          int      index);
 void br_queue_stop (BrQueue *queue);
 void br_queue_next (BrQueue *queue);
 void br_queue_add_uri (BrQueue    *queue,
@@ -125,10 +121,8 @@ void br_queue_get_name (BrQueue               *queue,
                         BrQueueGetNameCallback cb,
                         gpointer               userdata);
 void br_queue_set_position (BrQueue    *queue,
-                            BrQueueType type,
                             double      position);
 void br_queue_get_position (BrQueue                   *queue,
-                            BrQueueType                type,
                             BrQueueGetPositionCallback cb,
                             gpointer                   userdata);
 void br_queue_get_playing (BrQueue                  *queue,
