@@ -18,6 +18,10 @@
 
 #include <stdio.h>
 
+#include <glib/gi18n.h>
+
+#include <libnotify/notify.h>
+
 #include "bognor-marshal.h"
 #include "bognor-queue.h"
 
@@ -36,13 +40,12 @@ enum {
 
 struct _BognorQueuePrivate {
     GQueue *play_queue;
+    GVolumeMonitor *volume_monitor;
 
     char *name;
     gboolean playing;
 
     BognorQueueItem *current_item;
-
-    gboolean can_play_visual;
 };
 
 #define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), BOGNOR_TYPE_QUEUE, BognorQueuePrivate))
@@ -57,13 +60,15 @@ static gboolean bognor_queue_play_uri (BognorQueue *queue,
                                        const char  *uri,
                                        const char  *mimetype,
                                        GError     **error);
+static gboolean bognor_queue_play_index (BognorQueue *queue,
+                                         int          index,
+                                         GError     **error);
 static gboolean bognor_queue_get_playing (BognorQueue *queue,
                                           gboolean    *playing,
                                           GError     **error);
 
 static gboolean bognor_queue_get_now_playing (BognorQueue *queue,
-                                              char       **audio_uri,
-                                              char       **visual_uri,
+                                              char       **uri,
                                               GError     **error);
 static gboolean bognor_queue_list_uris (BognorQueue *queue,
                                         char      ***uris,
@@ -86,11 +91,9 @@ static gboolean bognor_queue_get_name (BognorQueue *queue,
                                        char       **name,
                                        GError     **error);
 static gboolean bognor_queue_set_position (BognorQueue *queue,
-                                           int          type,
                                            double       position,
                                            GError     **error);
 static gboolean bognor_queue_get_position (BognorQueue *queue,
-                                           int          type,
                                            double      *position,
                                            GError     **error);
 
@@ -101,6 +104,10 @@ free_queue_item (BognorQueueItem *item)
 {
     g_free (item->uri);
     g_free (item->mimetype);
+
+    g_object_unref (item->file);
+    g_object_unref (item->monitor);
+
     g_slice_free (BognorQueueItem, item);
 }
 
@@ -116,11 +123,7 @@ bognor_queue_play (BognorQueue *queue,
     }
 
     if (priv->current_item) {
-        if (priv->current_item->type == AUDIO_TYPE) {
-            klass->set_audio_playing (queue, TRUE);
-        } else {
-            klass->set_visual_playing (queue, TRUE);
-        }
+        klass->set_playing (queue, TRUE);
     }
 
     priv->playing = TRUE;
@@ -135,15 +138,7 @@ bognor_queue_stop (BognorQueue *queue,
     BognorQueueClass *klass = BOGNOR_QUEUE_GET_CLASS (queue);
     BognorQueuePrivate *priv = queue->priv;
 
-    klass->set_audio_playing (queue, FALSE);
-    klass->set_visual_playing (queue, FALSE);
-
-#if 0
-    g_signal_emit (queue, signals[NOW_PLAYING], 0,
-                   NULL, AUDIO_TYPE);
-    g_signal_emit (queue, signals[NOW_PLAYING], 0,
-                   NULL, VISUAL_TYPE);
-#endif
+    klass->set_playing (queue, FALSE);
 
     priv->playing = FALSE;
     g_signal_emit (queue, signals[PLAYING_CHANGED], 0, FALSE);
@@ -158,6 +153,61 @@ bognor_queue_next (BognorQueue *queue,
     return TRUE;
 }
 
+static void
+queue_item_changed_cb (GFileMonitor     *monitor,
+                       GFile            *file,
+                       GFile            *other_file,
+                       GFileMonitorEvent event_type,
+                       BognorQueue      *queue)
+{
+    BognorQueuePrivate *priv = queue->priv;
+    GList *u;
+    int i;
+
+    g_print ("Got event %d for %s\n", event_type, g_file_get_uri (file));
+    if (event_type != G_FILE_MONITOR_EVENT_DELETED) {
+        return;
+    }
+
+    g_print ("Got delete for %s\n", g_file_get_uri (file));
+    /* Find the queue item */
+    for (u = priv->play_queue->head, i = 0; u; u = u->next, i++) {
+        BognorQueueItem *item = (BognorQueueItem *) u->data;
+
+        if (item->monitor == monitor) {
+            g_queue_delete_link (priv->play_queue, u);
+
+            g_signal_emit (queue, signals[REMOVED], 0, item->uri, i);
+            free_queue_item (item);
+            break;
+        }
+    }
+}
+
+static BognorQueueItem *
+queue_item_new (BognorQueue *queue,
+                const char  *uri,
+                const char  *mimetype)
+{
+    BognorQueueItem *item;
+    GError *error = NULL;
+
+    item = g_slice_new (BognorQueueItem);
+    item->uri = g_strdup (uri);
+    item->mimetype = g_strdup (mimetype);
+    item->file = g_file_new_for_uri (uri);
+    item->monitor = g_file_monitor_file (item->file, G_FILE_MONITOR_WATCH_MOUNTS,
+                                         NULL, &error);
+    if (error) {
+        g_warning ("Error watching %s: %s", uri, error->message);
+        g_error_free (error);
+    } else {
+        g_signal_connect (item->monitor, "changed",
+                          G_CALLBACK (queue_item_changed_cb), queue);
+    }
+    return item;
+}
+
 static gboolean
 bognor_queue_play_uri (BognorQueue *queue,
                        const char  *uri,
@@ -168,14 +218,7 @@ bognor_queue_play_uri (BognorQueue *queue,
     BognorQueuePrivate *priv = queue->priv;
     BognorQueueItem *item;
 
-    item = g_slice_new (BognorQueueItem);
-    item->uri = g_strdup (uri);
-    item->mimetype = g_strdup (mimetype);
-    if (g_str_has_prefix (item->mimetype, "audio/")) {
-        item->type = AUDIO_TYPE;
-    } else {
-        item->type = VISUAL_TYPE;
-    }
+    item = queue_item_new (queue, uri, mimetype);
 
     if (priv->current_item) {
         free_queue_item (priv->current_item);
@@ -183,30 +226,39 @@ bognor_queue_play_uri (BognorQueue *queue,
     priv->current_item = item;
 
     klass->set_uri (queue, priv->current_item);
-
-    if (item->type == VISUAL_TYPE &&
-        priv->can_play_visual == FALSE) {
-        klass->force_visual (queue);
-    }
-
-    if (item->type == VISUAL_TYPE) {
-        klass->set_visual_playing (queue, TRUE);
-        klass->set_audio_playing (queue, FALSE);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, uri, VISUAL_TYPE);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, "", AUDIO_TYPE);
-    } else if (item->type == AUDIO_TYPE) {
-        klass->set_visual_playing (queue, FALSE);
-        klass->set_audio_playing (queue, TRUE);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, uri, AUDIO_TYPE);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, "", VISUAL_TYPE);
-    }
+    klass->set_playing (queue, TRUE);
+    g_signal_emit (queue, signals[NOW_PLAYING], 0, uri);
 
     priv->playing = TRUE;
     klass->add_item_to_recent (queue, item);
     g_signal_emit (queue, signals[PLAYING_CHANGED], 0, TRUE);
+
     return TRUE;
 }
 
+static gboolean
+bognor_queue_play_index (BognorQueue *queue,
+                         int          index,
+                         GError     **error)
+{
+    BognorQueuePrivate *priv = queue->priv;
+
+    for (; index > 0; index--) {
+        BognorQueueItem *item = g_queue_pop_head (priv->play_queue);
+
+        if (item == NULL) {
+            g_warning ("Nothing left in queue: %d items to remove", index);
+            return TRUE;
+        }
+
+        g_signal_emit (queue, signals[REMOVED], 0, item->uri, 0);
+
+        free_queue_item (item);
+    }
+
+    bognor_queue_get_next_item (queue);
+}
+
 static gboolean
 bognor_queue_get_playing (BognorQueue *queue,
                           gboolean    *playing,
@@ -247,21 +299,15 @@ bognor_queue_list_uris (BognorQueue *queue,
 
 static gboolean
 bognor_queue_get_now_playing (BognorQueue *queue,
-                              char       **audio_uri,
-                              char       **visual_uri,
+                              char       **uri,
                               GError     **error)
 {
     BognorQueuePrivate *priv = queue->priv;
 
     if (priv->current_item == NULL) {
-        *audio_uri = NULL;
-        *visual_uri = NULL;
+        *uri = NULL;
     } else {
-        if (priv->current_item->type == AUDIO_TYPE) {
-            *audio_uri = g_strdup (priv->current_item->uri);
-        } else {
-            *visual_uri = g_strdup (priv->current_item->uri);
-        }
+        *uri = g_strdup (priv->current_item->uri);
     }
 
     return TRUE;
@@ -277,14 +323,7 @@ bognor_queue_add_uri (BognorQueue *queue,
     BognorQueueItem *item;
     gboolean empty;
 
-    item = g_slice_new (BognorQueueItem);
-    item->uri = g_strdup (uri);
-    item->mimetype = g_strdup (mimetype);
-    if (g_str_has_prefix (item->mimetype, "audio/")) {
-        item->type = AUDIO_TYPE;
-    } else {
-        item->type = VISUAL_TYPE;
-    }
+    item = queue_item_new (queue, uri, mimetype);
 
     g_queue_push_tail (priv->play_queue, item);
 
@@ -326,10 +365,8 @@ bognor_queue_remove_uri (BognorQueue *queue,
             g_queue_delete_link (priv->play_queue, l);
 
             g_signal_emit (queue, signals[REMOVED], 0, item->uri, position);
-            g_free (item->uri);
-            g_free (item->mimetype);
-            g_slice_free (BognorQueueItem, item);
 
+            free_queue_item (item);
             break;
         }
 
@@ -352,9 +389,8 @@ bognor_queue_remove (BognorQueue *queue,
     if (item) {
         g_print ("Removing %s from %d\n", item->uri, index);
         g_signal_emit (queue, signals[REMOVED], 0, item->uri, index);
-        g_free (item->uri);
-        g_free (item->mimetype);
-        g_slice_free (BognorQueueItem, item);
+
+        free_queue_item (item);
     }
 
     return TRUE;
@@ -371,16 +407,8 @@ bognor_queue_insert_uri (BognorQueue *queue,
     BognorQueueItem *item;
     gboolean empty;
 
-    item = g_slice_new (BognorQueueItem);
-    item->uri = g_strdup (uri);
-    item->mimetype = g_strdup (mimetype);
-    if (g_str_has_prefix (mimetype, "audio/")) {
-        item->type = AUDIO_TYPE;
-    } else {
-        item->type = VISUAL_TYPE;
-    }
+    item = queue_item_new (queue, uri, mimetype);
 
-    g_print ("Pushing %s to %d\n", uri, position);
     g_queue_push_nth (priv->play_queue, item, position);
 
     g_signal_emit (queue, signals[ADDED], 0, uri, position);
@@ -401,34 +429,24 @@ bognor_queue_get_name (BognorQueue *queue,
 
 static gboolean
 bognor_queue_set_position (BognorQueue *queue,
-                           int          type,
                            double       position,
                            GError     **error)
 {
     BognorQueueClass *klass = BOGNOR_QUEUE_GET_CLASS (queue);
 
-    if (type == AUDIO_TYPE) {
-        klass->set_audio_position (queue, position);
-    } else {
-        klass->set_visual_position (queue, position);
-    }
+    klass->set_position (queue, position);
 
     return TRUE;
 }
 
 static gboolean
 bognor_queue_get_position (BognorQueue *queue,
-                           int          type,
                            double      *position,
                            GError     **error)
 {
     BognorQueueClass *klass = BOGNOR_QUEUE_GET_CLASS (queue);
 
-    if (type == AUDIO_TYPE) {
-        klass->get_audio_position (queue, position);
-    } else {
-        klass->get_visual_position (queue, position);
-    }
+    klass->get_position (queue, position);
 
     return TRUE;
 }
@@ -472,9 +490,9 @@ bognor_queue_set_property (GObject      *object,
 
 static void
 bognor_queue_get_property (GObject    *object,
-                          guint       prop_id,
-                          GValue     *value,
-                          GParamSpec *pspec)
+                           guint       prop_id,
+                           GValue     *value,
+                           GParamSpec *pspec)
 {
     BognorQueue *self = (BognorQueue *) object;
 
@@ -517,17 +535,15 @@ bognor_queue_class_init (BognorQueueClass *klass)
                                          G_TYPE_FROM_CLASS (klass),
                                          G_SIGNAL_RUN_FIRST |
                                          G_SIGNAL_NO_RECURSE, 0, NULL, NULL,
-                                         bognor_marshal_VOID__STRING_INT,
-                                         G_TYPE_NONE, 2, G_TYPE_STRING,
-                                         G_TYPE_INT);
+                                         g_cclosure_marshal_VOID__STRING,
+                                         G_TYPE_NONE, 1, G_TYPE_STRING);
     signals[POSITION_CHANGED] = g_signal_new ("position-changed",
                                               G_TYPE_FROM_CLASS (klass),
                                               G_SIGNAL_RUN_FIRST |
                                               G_SIGNAL_NO_RECURSE,
                                               0, NULL, NULL,
-                                              bognor_marshal_VOID__INT_DOUBLE,
-                                              G_TYPE_NONE, 2, G_TYPE_INT,
-                                              G_TYPE_DOUBLE);
+                                              g_cclosure_marshal_VOID__DOUBLE,
+                                              G_TYPE_NONE, 1, G_TYPE_DOUBLE);
     signals[PLAYING_CHANGED] = g_signal_new ("playing-changed",
                                              G_TYPE_FROM_CLASS (klass),
                                              G_SIGNAL_RUN_FIRST |
@@ -537,14 +553,58 @@ bognor_queue_class_init (BognorQueueClass *klass)
                                              G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
+static void
+mount_removed_cb (GVolumeMonitor *monitor,
+                  GMount         *mount,
+                  BognorQueue    *queue)
+{
+    BognorQueuePrivate *priv = queue->priv;
+    GFile *root;
+    GList *u;
+    int position;
+
+    root = g_mount_get_root (mount);
+    if (root == NULL) {
+        g_warning ("Got NULL root for mount removed");
+        return;
+    }
+
+    position = 0;
+    for (u = priv->play_queue->head; u;) {
+        BognorQueueItem *item = (BognorQueueItem *) u->data;
+        char *relative;
+
+        relative = g_file_get_relative_path (root, item->file);
+        if (relative) {
+            GList *old = u;
+
+            u = u->next;
+            g_queue_delete_link (priv->play_queue, old);
+
+            g_signal_emit (queue, signals[REMOVED], 0, item->uri, position);
+            free_queue_item (item);
+            g_free (relative);
+        } else {
+            position++;
+            u = u->next;
+        }
+    }
+
+    g_object_unref (root);
+}
+
 static void
 bognor_queue_init (BognorQueue *self)
 {
     BognorQueuePrivate *priv;
 
     priv = self->priv = GET_PRIVATE (self);
+
+    priv->volume_monitor = g_volume_monitor_get ();
+    g_signal_connect (priv->volume_monitor, "mount-removed",
+                      G_CALLBACK (mount_removed_cb), self);
+
     priv->play_queue = g_queue_new ();
-    priv->can_play_visual = FALSE;
 }
 
 BognorQueue *
@@ -574,44 +634,6 @@ bognor_queue_new_from_playlist (const char *playlist)
     return q;
 }
 
-void
-bognor_queue_set_visual_enabled (BognorQueue *queue,
-                                 gboolean     enabled)
-{
-    BognorQueuePrivate *priv = queue->priv;
-    BognorQueueClass *klass = BOGNOR_QUEUE_GET_CLASS (queue);
-    BognorQueueItem *item;
-
-    priv->can_play_visual = enabled;
-
-    item = bognor_queue_get_current_item (queue);
-    if (item == NULL) {
-        return;
-    }
-
-    if (priv->can_play_visual == FALSE) {
-        if (item->type == VISUAL_TYPE) {
-            klass->set_visual_playing (queue, FALSE);
-            priv->playing = FALSE;
-            g_signal_emit (queue, signals[PLAYING_CHANGED], 0, FALSE);
-        }
-    } else {
-        if (item->type == VISUAL_TYPE) {
-            klass->set_visual_playing (queue, TRUE);
-            priv->playing = TRUE;
-            g_signal_emit (queue, signals[PLAYING_CHANGED], 0, TRUE);
-        }
-    }
-}
-
-gboolean
-bognor_queue_get_visual_enabled (BognorQueue *queue)
-{
-    BognorQueuePrivate *priv = queue->priv;
-
-    return priv->can_play_visual;
-}
-
 void
 bognor_queue_set_name (BognorQueue *queue,
                        const char  *name)
@@ -623,10 +645,9 @@ bognor_queue_set_name (BognorQueue *queue,
 
 void
 bognor_queue_emit_position_changed (BognorQueue *queue,
-                                    double       position,
-                                    int          type)
+                                    double       position)
 {
-    g_signal_emit (queue, signals[POSITION_CHANGED], 0, type, position);
+    g_signal_emit (queue, signals[POSITION_CHANGED], 0, position);
 }
 
 BognorQueueItem *
@@ -643,27 +664,8 @@ bognor_queue_get_next_item (BognorQueue *queue)
     BognorQueueClass *klass = BOGNOR_QUEUE_GET_CLASS (queue);
     BognorQueuePrivate *priv = queue->priv;
     BognorQueueItem *item = NULL;
-    GList *l;
-    int position = 0;
 
-    for (l = priv->play_queue->head; l; l = l->next) {
-        BognorQueueItem *i = l->data;
-
-        if (i->type == AUDIO_TYPE) {
-            g_queue_delete_link (priv->play_queue, l);
-            item = i;
-            break;
-        }
-
-        if (i->type == VISUAL_TYPE &&
-            priv->can_play_visual == TRUE) {
-            g_queue_delete_link (priv->play_queue, l);
-            item = i;
-            break;
-        }
-
-        position++;
-    }
+    item = g_queue_pop_head (priv->play_queue);
 
     if (priv->current_item) {
         free_queue_item (priv->current_item);
@@ -672,18 +674,13 @@ bognor_queue_get_next_item (BognorQueue *queue)
 
     klass->set_uri (queue, priv->current_item);
     if (item) {
-        g_signal_emit (queue, signals[REMOVED], 0, item->uri, position);
+        g_signal_emit (queue, signals[REMOVED], 0, item->uri, 0);
         klass->add_item_to_recent (queue, item);
     }
     if (item == NULL) {
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, NULL, AUDIO_TYPE);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, NULL, VISUAL_TYPE);
-    } else if (item->type == AUDIO_TYPE) {
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, item->uri, item->type);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, NULL, VISUAL_TYPE);
-    } else if (item->type == VISUAL_TYPE) {
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, item->uri, item->type);
-        g_signal_emit (queue, signals[NOW_PLAYING], 0, NULL, AUDIO_TYPE);
+        g_signal_emit (queue, signals[NOW_PLAYING], 0, NULL);
+    } else {
+        g_signal_emit (queue, signals[NOW_PLAYING], 0, item->uri);
     }
 
     return item;
@@ -696,16 +693,9 @@ bognor_queue_play_next (BognorQueue *queue)
     BognorQueueItem *item = NULL;
 
     if (item = bognor_queue_get_next_item (queue)) {
-        if (item->type == VISUAL_TYPE) {
-            klass->set_audio_playing (queue, FALSE);
-            klass->set_visual_playing (queue, TRUE);
-        } else {
-            klass->set_audio_playing (queue, TRUE);
-            klass->set_visual_playing (queue, FALSE);
-        }
+        klass->set_playing (queue, TRUE);
     } else {
-        klass->set_audio_playing (queue, FALSE);
-        klass->set_visual_playing (queue, FALSE);
+        klass->set_playing (queue, FALSE);
     }
 }
 
@@ -716,3 +706,29 @@ bognor_queue_get_is_playing (BognorQueue *queue)
 
     return priv->playing;
 }
+
+void
+bognor_queue_notify_unknown_format (BognorQueue *queue,
+                                    const char  *uri)
+{
+    NotifyNotification *notification;
+    char *basename, *unesc, *message;
+
+    basename = g_path_get_basename (uri);
+    unesc = g_uri_unescape_string (basename, NULL);
+    g_free (basename);
+
+#ifdef ENABLE_HELIX
+    message = g_strdup_printf (_("Sorry, we can't play %s, as we don't have the correct plugin."), unesc);
+#else
+    message = g_strdup_printf (_("Sorry, we can't play %s, as we don't have the correct plugin. You could try searching for ❝gstreamer codecs❞ on the web."), unesc);
+#endif
+    g_free (unesc);
+
+    notification = notify_notification_new (_("We can't play this"),
+                                            message, NULL, NULL);
+    notify_notification_show (notification, NULL);
+    g_object_unref (notification);
+
+    g_free (message);
+}
