@@ -20,7 +20,7 @@
 #define __BOGNOR_QUEUE_H__
 
 #include <glib-object.h>
-
+#include <gio/gio.h>
 
 G_BEGIN_DECLS
 
@@ -49,15 +49,11 @@ typedef struct _BognorQueuePrivate BognorQueuePrivate;
 typedef struct _BognorQueue      BognorQueue;
 typedef struct _BognorQueueClass BognorQueueClass;
 
-typedef enum _BognorQueueItemType {
-    AUDIO_TYPE,
-    VISUAL_TYPE,
-} BognorQueueItemType;
-
 typedef struct _BognorQueueItem {
     char *uri;
     char *mimetype;
-    BognorQueueItemType type;
+    GFile *file;
+    GFileMonitor *monitor;
 } BognorQueueItem;
 
 struct _BognorQueue {
@@ -72,20 +68,12 @@ struct _BognorQueueClass {
     gboolean (*set_uri) (BognorQueue      *queue,
                          BognorQueueItem *item);
 
-    gboolean (*set_audio_playing) (BognorQueue *queue,
-                                   gboolean     playing);
-    gboolean (*set_visual_playing) (BognorQueue *queue,
-                                    gboolean     playing);
-
-    gboolean (*set_audio_position) (BognorQueue *queue,
-                                    double       position);
-    gboolean (*get_audio_position) (BognorQueue *queue,
-                                    double      *position);
-    gboolean (*set_visual_position) (BognorQueue *queue,
-                                     double       position);
-    gboolean (*get_visual_position) (BognorQueue *queue,
-                                     double      *position);
-    gboolean (*force_visual) (BognorQueue *queue);
+    gboolean (*set_playing) (BognorQueue *queue,
+                             gboolean     playing);
+    gboolean (*set_position) (BognorQueue *queue,
+                              double       position);
+    gboolean (*get_position) (BognorQueue *queue,
+                              double      *position);
     gboolean (*add_item_to_recent) (BognorQueue     *queue,
                                     BognorQueueItem *item);
 };
@@ -95,19 +83,17 @@ GType bognor_queue_get_type (void) G_GNUC_CONST;
 gboolean bognor_queue_play (BognorQueue *queue,
                             GError     **error);
 
-void bognor_queue_set_visual_enabled (BognorQueue *queue,
-                                      gboolean     enabled);
-gboolean bognor_queue_get_visual_enabled (BognorQueue *queue);
 void bognor_queue_set_name (BognorQueue *queue,
                             const char  *name);
 void bognor_queue_emit_position_changed (BognorQueue *queue,
-                                         double       position,
-                                         int          type);
+                                         double       position);
 
 BognorQueueItem *bognor_queue_get_current_item (BognorQueue *queue);
 BognorQueueItem *bognor_queue_get_next_item (BognorQueue *queue);
 void bognor_queue_play_next (BognorQueue *queue);
 gboolean bognor_queue_get_is_playing (BognorQueue *queue);
+void bognor_queue_notify_unknown_format (BognorQueue *queue,
+                                         const char  *uri);
 
 G_END_DECLS
 
