@@ -22,13 +22,17 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#ifdef ENABLE_GST
 #include <gst/gst.h>
+#endif
 #include <gtk/gtk.h>
 
 #include <dbus/dbus-protocol.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-bindings.h>
 
+#include <libnotify/notify.h>
+
 #include "bognor-queue-manager.h"
 
 typedef struct BognorRegis {
@@ -90,7 +94,9 @@ main (int    argc,
       char **argv)
 {
     g_type_init ();
+#ifdef ENABLE_GST
     gst_init (&argc, &argv);
+#endif
     gtk_init (&argc, &argv);
 
     br = g_new0 (BognorRegis, 1);
@@ -99,6 +105,8 @@ main (int    argc,
         return 0;
     }
 
+    notify_init ("Bognor-Regis Media Daemon");
+
     br->mainloop = g_main_loop_new (NULL, FALSE);
     g_main_loop_run (br->mainloop);
 
